// This file is part of BUDEES, a software that allows fast simulation of bubble dynamics in electrolyzers and their interactions with electrical parameters. 
// 
// Copyright (C) 2024 <CNRS-ENS Rennes-Université de Rennes-CY Cergy Paris Université-Université Paris Saclay-CNAM Paris-ENS Paris-Saclay>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with this program. If not, see https://www.gnu.org/licenses/



/* --- Compute the extraction proportions --- */
/* ------------------------------------------ */
void updateRemaining(ElectrolysisStatistics& Statistics, ListBubbles& Bubbles) {
	Statistics.remaining[0] = 0; Statistics.remaining[1] = 0;
	for (list<Bubble>::iterator it = Bubbles.attached.begin(); it != Bubbles.attached.end(); it++) {
		Statistics.remaining[0] += it->quantity[0];
		Statistics.remaining[1] += it->quantity[1];
	}
	for (list<Bubble>::iterator it = Bubbles.detached.begin(); it != Bubbles.detached.end(); it++) {
		Statistics.remaining[0] += it->quantity[0];
		Statistics.remaining[1] += it->quantity[1];
	}
}
/* ------------------------------------------ */


/* ---------- Compute performances ---------- */
/* ------------------------------------------ */
Performances zeroPerformances() {
	return {{1e9, 1e9}, {0.0, 0.0}, {0.0, 0.0}, {1e9, 1e9}, {0.0, 0.0}, 1e9};
}
Performances computePerformances(Parameters& parameters, ElectrolysisStatistics& Statistics, ListBubbles& Bubbles) {
	updateRemaining(Statistics, Bubbles);
	Performances performances = zeroPerformances();

	for (int s = 0; s < 2; s++) {
		performances.electrolysed[s] = (parameters.I*parameters.tf) / QnPrefactor[s];
		performances.collected[s] = Statistics.collected[s];
		performances.contamination[s] = Statistics.contamination[s];
		performances.lost[s] = Statistics.lost[s];
		performances.remaining[s] = Statistics.remaining[s];
	}

	performances.resistor = Statistics.resistor;

	return performances;
}

void updateStatistics(ElectrolysisStatistics& Statistics, MatrixTension& TENSION, MatrixXd& CURRENT) {
	Statistics.resistor = TENSION.resistor;
	Statistics.overpotential = (TENSION.phi_anode-TENSION.phi_cathode);
	Statistics.tension[0] = TENSION.min; Statistics.tension[1] = TENSION.max;
	Statistics.current[0] = CURRENT.minCoeff(); Statistics.current[1] = CURRENT.maxCoeff();
}
/* ------------------------------------------ */
