// This file is part of BUDEES, a software that allows fast simulation of bubble dynamics in electrolyzers and their interactions with electrical parameters. 
// 
// Copyright (C) 2024 <CNRS-ENS Rennes-Université de Rennes-CY Cergy Paris Université-Université Paris Saclay-CNAM Paris-ENS Paris-Saclay>
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License along with this program. If not, see https://www.gnu.org/licenses/

/* ------ Test bubble intractability ------ */
/* ---------------------------------------- */
void testInteractibilityMode(Bubble* it, Bubble* jt, int i, int j, int modeI, int modeJ, double deltaX, double deltaY, double RavvSq, double dMax, bool* stopFlag) {
	if ((mathAbs(deltaX) < dMax) && (mathAbs(deltaY) < dMax)) {
		double deltaSqNorm = (deltaX*deltaX + deltaY*deltaY) / RavvSq; // Distance inter-bulle normalisée par le rayon moyen
		if (deltaSqNorm < interactionLongMaximumRangeSq) {
			if (it->interactor.count == interactionMaxAmountAbsolute) {*stopFlag = true; return;}
			it->interactor.interactionIndexs[it->interactor.count] = j;
			it->interactor.distancesSqNorm[it->interactor.count] = deltaSqNorm;
			it->interactor.bubbles[it->interactor.count] = jt;
			it->interactor.imagedModes[it->interactor.count] = modeJ;
			it->interactor.count++;

			if (jt->interactor.count == interactionMaxAmountAbsolute) {*stopFlag = true; return;}
			jt->interactor.interactionIndexs[jt->interactor.count] = i;
			jt->interactor.distancesSqNorm[jt->interactor.count] = deltaSqNorm;
			jt->interactor.bubbles[jt->interactor.count] = it;
			jt->interactor.imagedModes[jt->interactor.count] = modeI;
			jt->interactor.count++;
		}
	}
}
void checkBubbles(list<Bubble>::iterator& it, int i, ListBubbles& Bubbles, double Ly, bool* stopFlag) {
	it->statCollision = 0;
	int j = i; list<Bubble>::iterator itbis = it;
	// Bulles détachées
	for (itbis++; itbis != Bubbles.detached.end(); itbis++) {
		j++;
		double Ravv = 0.5*(it->r + itbis->r), RavvSq = SQ(Ravv);
		double dMax = Ravv * interactionLongMaximumRange;

		/* mode 0 : pas de modifications */
		double deltaX = (itbis->x - it->x), deltaY = (itbis->y - it->y);
		testInteractibilityMode(&*it, &*itbis, i, j, 0, 0, deltaX, deltaY, RavvSq, dMax, stopFlag);
		/* mode 1 : recopie en haut */
		if (itbis->y < dMax) {
			deltaX = (itbis->x - it->x), deltaY = ((itbis->y + Ly) - it->y);
			testInteractibilityMode(&*it, &*itbis, i, j, 2, 1, deltaX, deltaY, RavvSq, dMax, stopFlag);
		}
		/* mode 2 : recopie en bas */
		if (itbis->y > (Ly - dMax)) {
			deltaX = (itbis->x - it->x), deltaY = ((itbis->y - Ly) - it->y);
			testInteractibilityMode(&*it, &*itbis, i, j, 1, 2, deltaX, deltaY, RavvSq, dMax, stopFlag);
		}
	}
	// Bulles attachées
	for (itbis = Bubbles.attached.begin(); itbis != Bubbles.attached.end(); itbis++) {
		double Ravv = 0.5*(it->r + itbis->r), RavvSq = SQ(Ravv);
		double dMax = Ravv * interactionLongMaximumRange;

		/* mode 0 : pas de modifications */
		double deltaX = (itbis->x - it->x), deltaY = (itbis->y - it->y);
		testInteractibilityMode(&*it, &*itbis, i, -1, 0, 0, deltaX, deltaY, RavvSq, dMax, stopFlag);
		/* mode 1 : recopie en haut */
		if (itbis->y < dMax) {
			deltaX = (itbis->x - it->x), deltaY = ((itbis->y + Ly) - it->y);
			testInteractibilityMode(&*it, &*itbis, i, -1, 2, 1, deltaX, deltaY, RavvSq, dMax, stopFlag);
		}
		/* mode 2 : recopie en bas */
		if (itbis->y > (Ly - dMax)) {
			deltaX = (itbis->x - it->x), deltaY = ((itbis->y - Ly) - it->y);
			testInteractibilityMode(&*it, &*itbis, i, -1, 1, 2, deltaX, deltaY, RavvSq, dMax, stopFlag);
		}
	}
}

/* Correct y for repeating symmetry */
double yRepeat(double y, double Ly, int mode) {
	if (mode == 0) return y; // mode 0
	else if (mode == 1) return y+Ly; // mode 1
	else return y-Ly; // mode 2
}
/* ---------------------------------------- */
